==> daemonshogi-0.1.3.tar.gz: daemonshogi-0.1.3/src/si/moveinfo.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/*
 * moveinfo.c
 * $Source: /cvs/si/moveinfo.c,v $
 * $Id: moveinfo.c,v 1.37 2002/09/11 21:17:27 mik Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "si.h"

/**
 * ݤMOVEINFOΥݥ󥿤֤
 * @return MOVEINFOΥݥ
 */
MOVEINFO *newMOVEINFO(void) {
  MOVEINFO *mi;
  mi = (MOVEINFO *) malloc(sizeof(MOVEINFO));
  if (mi == NULL) {
    si_abort("No enough memory. In moveinfo.c#newMOVEFINO()");
  }
  mi->count = 0;

  return mi;
}

/**
 * mi Υ롣
 * @param mi оݤMOVEINFO
 */
void freeMOVEINFO(MOVEINFO *mi) {
#ifdef DEBUG
  assert(mi != NULL);
#endif /* DEBUG */
  
  free( mi );
} 

#ifndef USE_MISC_MACRO
/**
 * MOVEINFO  TE ɲä롣
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAdd(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);

  if ( te->uti ) {
    assert(te->fm == 0);
    assert(FU <= te->uti && te->uti <= HISYA);
  } else {
    assert(1 <= (te->fm & 0xf) && (te->fm & 0xf) <= 9);
    assert(1 <= (te->fm >> 4) && (te->fm >> 4) <= 9);
  }
  assert(1 <= (te->to & 0xf) && (te->to & 0xf) <= 9);
  assert(1 <= (te->to >> 4) && (te->to >> 4) <= 9);

  /* out of bounds check */
  assert(mi->count < MOVEINFOMAX);
#endif /* DEBUG */
  
  if ( MOVEINFOMAX <= mi->count )
    return;

  mi->te[mi->count++] = *te;
}
#endif /* USE_MISC_MACRO */

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣
 * @param mi оݤmi
 * @param te ɲäte
 * @param next 
 */
void miAddWithNari(MOVEINFO * mi, TE * te, int next) {
#ifdef NOREC_NARAZU
  int p;
  extern BOARD g_board;
#endif /* NOREC_NARAZU */
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
  assert(1 <= (te->fm >> 4) && (te->fm >> 4) <= 9);
  assert(1 <= (te->fm & 0xF) && (te->fm & 0xF) <= 9);
  assert(1 <= (te->to >> 4) && (te->to >> 4) <= 9);
  assert(1 <= (te->to & 0xF) && (te->to & 0xF) <= 9);
  assert(next == 0 || next == 1);
#endif /* DEBUG */

#ifdef NOREC_NARAZU
  p = boGetPiece(&g_board, te->fm) & 0xF;
  if ( next == GOTE ) {
    if ( (p == FU || p == KAKU || p == HISYA) ) {
      if ( 0x70 <= te->fm || 0x70 <= te->to ) {
	te->nari = 1;
	miAdd(mi, te);
      } else {
	te->nari = 0;
	miAdd(mi, te);
      }
    } else {
      /* ⡢ѡְʳ */
      if ( 0x70 <= te->fm || 0x70 <= te->to ) {
	te->nari = 1;
	miAdd(mi, te);
      }
      te->nari = 0;
      miAdd(mi, te);
    }
  } else {
    if ( (p == FU || p == KAKU || p == HISYA) ) {
      if ( te->fm <= 0x39 || te->to <= 0x39 ) {
	te->nari = 1;
	miAdd(mi, te);
      } else {
	te->nari = 0;
	miAdd(mi, te);
      }
    } else {
      /* ⡢ѡְʳ */
      if ( te->fm <= 0x39 || te->to <= 0x39 ) {
	te->nari = 1;
	miAdd(mi, te);
      }
      te->nari = 0;
      miAdd(mi, te);
    }
  }
#else
  if (next) { /* gote */
    if (0x70 <= te->fm || 0x70 <= te->to) {
      te->nari = 1;
      miAdd(mi, te);
    }
  } else { /* sente */
    if (te->fm <= 0x39 || te->to <= 0x39) {
      te->nari = 1;
      miAdd(mi, te);
    }
  }
  te->nari = 0;
  miAdd(mi, te);
#endif /* NOREC_NARAZU */
}

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣ѡ
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAddWithNariSente(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
#endif /* DEBUG */

  if (te->fm <= 0x39 || te->to <= 0x39) {
    te->nari = 1;
    miAdd(mi, te);
  }

  te->nari = 0;
  miAdd(mi, te);
}

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣ѡ
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAddWithNariGote(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
#endif /* DEBUG */

  if (0x70 <= te->fm || 0x70 <= te->to) {
    te->nari = 1;
    miAdd(mi, te);
  }
    
  te->nari = 0;
  miAdd(mi, te);
}

/**
 * MOVEINFO  TE ä롣
 * 餺ǤʤϵϿʤ
 * 
 * @param mi MOVEINFO
 * @param te Ͽ te
 * @param p Ͽ te ζ
 * @param next μ
 */
void miAddWithNari2(MOVEINFO * mi, TE te, int p, int next) {
  int xy;
  int w;
  
#ifdef DEBUG
  assert(mi != NULL);
  assert(1 <= (te.fm >> 4) && (te.fm >> 4) <= 9);
  assert(1 <= (te.fm & 0xF) && (te.fm & 0xF) <= 9);
  assert(1 <= (te.to >> 4) && (te.to >> 4) <= 9);
  assert(1 <= (te.to & 0xF) && (te.to & 0xF) <= 9);
  assert(next == SENTE || next == GOTE);
  assert(p != 0 && p != WALL);
#endif /* DEBUG */

  xy = te.to;
  w = (xy >> 4);

  /** ⡢ϤΤޤ miAdd()  */
  if ( p == KIN || p == OH || (p & 8) ) {
    miAdd(mi, &te);
    return;
  }

  if ( p == FU || p == KYO ) {
    if ( next ) {
      if ( w == 9 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    } else {
      if ( w == 1 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    }
  } else if ( p == KEI ) {
    if ( next == GOTE ) {
      if ( 8 <= w ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    } else {
      if ( w <= 2 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    }
  } else {
    miAddWithNari(mi, &te, next);
  }
}

/**
 * mi ˳ǼƤ index ܤ te ؤΥݥ󥿤֤
 * @param mi оݤMOVEINFO
 * @param index ݥ󥿤Фֹ
 * @return teؤΥݥ
 */
#ifndef USE_MISC_MACRO
TE *miGet(MOVEINFO *mi, int index) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(0 <= index && index < mi->count); 
#endif /* DEBUG */
  return &(mi->te[index]);
}
#endif /* USE_MISC_MACRO */

/**
 * miteƴ֤ͤ롣
 * @param mi оݤMOVEINFO
 * @param te оݤTE
 */
void miRemoveTE(MOVEINFO *mi, TE *te ) {
  int i;

#ifdef DEBUG
  assert( mi != NULL );
  assert( te != NULL );
#endif /* DEBUG */
  
  for (i=0; i<mi->count; i++) {
    if ( mi->te[ i ].fm == te->fm &&
	mi->te[ i ].to == te->to &&
	mi->te[ i ].nari == te->nari &&
	mi->te[ i ].uti == te->uti ) {
      if ( mi->count - 1 == i ) {
	mi->count--;
	return;
      }
      
      mi->te[ i ] = mi->te[ mi->count - 1 ];
      mi->count--;
      return;
    }
  }
}

/**
 * ֹꤷmiteƴ֤ͤ롣
 * @param mi оݤMOVEINFO
 * @param num ֹ
 */
void miRemoveTE2(MOVEINFO *mi, int num ) {
#ifdef DEBUG
  assert( mi != NULL );
  assert( 0 <= num && num <= mi->count );
#endif /* DEBUG */
  
  if ( mi->count - 1 == num ) {
    mi->count--;
    return;
  }
      
  mi->te[ num ] = mi->te[ mi->count - 1 ];
  mi->count--;
  return;
}

/**
 * miʣ롣ȤϿ줿Ĥ
 * @param mi оݤMOVEINFO
 */ 
void miRemoveDuplicationTE(MOVEINFO *mi) {
  int i, j;

#ifdef DEBUG
  assert( mi != NULL );
#endif /* DEBUG */

  for ( i=0; i<mi->count; i++ ) {
    for ( j=i+1; j<mi->count; ) {
      if ( ! teCmpTE( &(mi->te[ i ]), &(mi->te[ j ]) ) ) {
	miRemoveTE2( mi, j );
      } else {
	j++;
      }
    }
  }
}

/**
 * src Ƥ dst إԡ롣
 * @param src ԡ
 * @param dst ԡ
 */
void miCopy(MOVEINFO *dest, MOVEINFO *src) {
#ifdef DEBUG
  assert(dest != NULL);
  assert(src != NULL);
  assert(0 <= src->count && src->count <= MOVEINFOMAX);
#endif /* DEBUG */
  memcpy(dest, src, sizeof(MOVEINFO));
}

==> daemonshogi-0.1.4.tar.gz: daemonshogi-0.1.4/src/si/moveinfo.c <==
/* SI SHOGI Library
   Copyright (C) 2002 Masahiko Tokita
   This file is part of the SI SHOGI Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.w

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with The Daemonshogi; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */
/*
 * moveinfo.c
 * $Source: /cvsroot/daemonshogi/daemonshogi/src/si/moveinfo.c,v $
 * $Id: moveinfo.c,v 1.1.1.1 2005/12/09 09:03:09 tokita Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "si.h"

/**
 * ݤMOVEINFOΥݥ󥿤֤
 * @return MOVEINFOΥݥ
 */
MOVEINFO *newMOVEINFO(void) {
  MOVEINFO *mi;
  mi = (MOVEINFO *) malloc(sizeof(MOVEINFO));
  if (mi == NULL) {
    si_abort("No enough memory. In moveinfo.c#newMOVEFINO()");
  }
  mi->count = 0;

  return mi;
}

/**
 * mi Υ롣
 * @param mi оݤMOVEINFO
 */
void freeMOVEINFO(MOVEINFO *mi) {
#ifdef DEBUG
  assert(mi != NULL);
#endif /* DEBUG */
  
  free( mi );
} 

#ifndef USE_MISC_MACRO
/**
 * MOVEINFO  TE ɲä롣
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAdd(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);

  if ( te->uti ) {
    assert(te->fm == 0);
    assert(FU <= te->uti && te->uti <= HISYA);
  } else {
    assert(1 <= (te->fm & 0xf) && (te->fm & 0xf) <= 9);
    assert(1 <= (te->fm >> 4) && (te->fm >> 4) <= 9);
  }
  assert(1 <= (te->to & 0xf) && (te->to & 0xf) <= 9);
  assert(1 <= (te->to >> 4) && (te->to >> 4) <= 9);

  /* out of bounds check */
  assert(mi->count < MOVEINFOMAX);
#endif /* DEBUG */
  
  if ( MOVEINFOMAX <= mi->count )
    return;

  mi->te[mi->count++] = *te;
}
#endif /* USE_MISC_MACRO */

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣
 * @param mi оݤmi
 * @param te ɲäte
 * @param next 
 */
void miAddWithNari(MOVEINFO * mi, TE * te, int next) {
#ifdef NOREC_NARAZU
  int p;
  extern BOARD g_board;
#endif /* NOREC_NARAZU */
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
  assert(1 <= (te->fm >> 4) && (te->fm >> 4) <= 9);
  assert(1 <= (te->fm & 0xF) && (te->fm & 0xF) <= 9);
  assert(1 <= (te->to >> 4) && (te->to >> 4) <= 9);
  assert(1 <= (te->to & 0xF) && (te->to & 0xF) <= 9);
  assert(next == 0 || next == 1);
#endif /* DEBUG */

#ifdef NOREC_NARAZU
  p = boGetPiece(&g_board, te->fm) & 0xF;
  if ( next == GOTE ) {
    if ( (p == FU || p == KAKU || p == HISYA) ) {
      if ( 0x70 <= te->fm || 0x70 <= te->to ) {
	te->nari = 1;
	miAdd(mi, te);
      } else {
	te->nari = 0;
	miAdd(mi, te);
      }
    } else {
      /* ⡢ѡְʳ */
      if ( 0x70 <= te->fm || 0x70 <= te->to ) {
	te->nari = 1;
	miAdd(mi, te);
      }
      te->nari = 0;
      miAdd(mi, te);
    }
  } else {
    if ( (p == FU || p == KAKU || p == HISYA) ) {
      if ( te->fm <= 0x39 || te->to <= 0x39 ) {
	te->nari = 1;
	miAdd(mi, te);
      } else {
	te->nari = 0;
	miAdd(mi, te);
      }
    } else {
      /* ⡢ѡְʳ */
      if ( te->fm <= 0x39 || te->to <= 0x39 ) {
	te->nari = 1;
	miAdd(mi, te);
      }
      te->nari = 0;
      miAdd(mi, te);
    }
  }
#else
  if (next) { /* gote */
    if (0x70 <= te->fm || 0x70 <= te->to) {
      te->nari = 1;
      miAdd(mi, te);
    }
  } else { /* sente */
    if (te->fm <= 0x39 || te->to <= 0x39) {
      te->nari = 1;
      miAdd(mi, te);
    }
  }
  te->nari = 0;
  miAdd(mi, te);
#endif /* NOREC_NARAZU */
}

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣ѡ
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAddWithNariSente(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
#endif /* DEBUG */

  if (te->fm <= 0x39 || te->to <= 0x39) {
    te->nari = 1;
    miAdd(mi, te);
  }

  te->nari = 0;
  miAdd(mi, te);
}

/**
 * MOVEINFO  TE ɲä롣⵭Ͽ롣ѡ
 * @param mi оݤmi
 * @param te ɲäte
 */
void miAddWithNariGote(MOVEINFO * mi, TE * te) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(te != NULL);
#endif /* DEBUG */

  if (0x70 <= te->fm || 0x70 <= te->to) {
    te->nari = 1;
    miAdd(mi, te);
  }
    
  te->nari = 0;
  miAdd(mi, te);
}

/**
 * MOVEINFO  TE ä롣
 * 餺ǤʤϵϿʤ
 * 
 * @param mi MOVEINFO
 * @param te Ͽ te
 * @param p Ͽ te ζ
 * @param next μ
 */
void miAddWithNari2(MOVEINFO * mi, TE te, int p, int next) {
  int xy;
  int w;
  
#ifdef DEBUG
  assert(mi != NULL);
  assert(1 <= (te.fm >> 4) && (te.fm >> 4) <= 9);
  assert(1 <= (te.fm & 0xF) && (te.fm & 0xF) <= 9);
  assert(1 <= (te.to >> 4) && (te.to >> 4) <= 9);
  assert(1 <= (te.to & 0xF) && (te.to & 0xF) <= 9);
  assert(next == SENTE || next == GOTE);
  assert(p != 0 && p != WALL);
#endif /* DEBUG */

  xy = te.to;
  w = (xy >> 4);

  /** ⡢ϤΤޤ miAdd()  */
  if ( p == KIN || p == OH || (p & 8) ) {
    miAdd(mi, &te);
    return;
  }

  if ( p == FU || p == KYO ) {
    if ( next ) {
      if ( w == 9 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    } else {
      if ( w == 1 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    }
  } else if ( p == KEI ) {
    if ( next == GOTE ) {
      if ( 8 <= w ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    } else {
      if ( w <= 2 ) {
	te.nari = 1;
	miAdd(mi, &te);
      } else {
	miAddWithNari(mi, &te, next);
      }
    }
  } else {
    miAddWithNari(mi, &te, next);
  }
}

/**
 * mi ˳ǼƤ index ܤ te ؤΥݥ󥿤֤
 * @param mi оݤMOVEINFO
 * @param index ݥ󥿤Фֹ
 * @return teؤΥݥ
 */
#ifndef USE_MISC_MACRO
TE *miGet(MOVEINFO *mi, int index) {
#ifdef DEBUG
  assert(mi != NULL);
  assert(0 <= index && index < mi->count); 
#endif /* DEBUG */
  return &(mi->te[index]);
}
#endif /* USE_MISC_MACRO */

/**
 * miteƴ֤ͤ롣
 * @param mi оݤMOVEINFO
 * @param te оݤTE
 */
void miRemoveTE(MOVEINFO *mi, TE *te ) {
  int i;

#ifdef DEBUG
  assert( mi != NULL );
  assert( te != NULL );
#endif /* DEBUG */
  
  for (i=0; i<mi->count; i++) {
    if ( mi->te[ i ].fm == te->fm &&
	mi->te[ i ].to == te->to &&
	mi->te[ i ].nari == te->nari &&
	mi->te[ i ].uti == te->uti ) {
      if ( mi->count - 1 == i ) {
	mi->count--;
	return;
      }
      
      mi->te[ i ] = mi->te[ mi->count - 1 ];
      mi->count--;
      return;
    }
  }
}

/**
 * ֹꤷmiteƴ֤ͤ롣
 * @param mi оݤMOVEINFO
 * @param num ֹ
 */
void miRemoveTE2(MOVEINFO *mi, int num ) {
#ifdef DEBUG
  assert( mi != NULL );
  assert( 0 <= num && num <= mi->count );
#endif /* DEBUG */
  
  if ( mi->count - 1 == num ) {
    mi->count--;
    return;
  }
      
  mi->te[ num ] = mi->te[ mi->count - 1 ];
  mi->count--;
  return;
}

/**
 * miʣ롣ȤϿ줿Ĥ
 * @param mi оݤMOVEINFO
 */ 
void miRemoveDuplicationTE(MOVEINFO *mi) {
  int i, j;

#ifdef DEBUG
  assert( mi != NULL );
#endif /* DEBUG */

  for ( i=0; i<mi->count; i++ ) {
    for ( j=i+1; j<mi->count; ) {
      if ( ! teCmpTE( &(mi->te[ i ]), &(mi->te[ j ]) ) ) {
	miRemoveTE2( mi, j );
      } else {
	j++;
      }
    }
  }
}

/**
 * src Ƥ dst إԡ롣
 * @param src ԡ
 * @param dst ԡ
 */
void miCopy(MOVEINFO *dest, MOVEINFO *src) {
#ifdef DEBUG
  assert(dest != NULL);
  assert(src != NULL);
  assert(0 <= src->count && src->count <= MOVEINFOMAX);
#endif /* DEBUG */
  memcpy(dest, src, sizeof(MOVEINFO));
}
